# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.cluster import Cluster
from ..types.cluster_manifest import ClusterManifest
from ..types.get_cluster_response import GetClusterResponse
from ..types.is_cluster_connected_response import IsClusterConnectedResponse
from ..types.list_cluster_addons_response import ListClusterAddonsResponse
from ..types.list_clusters_response import ListClustersResponse
from .raw_client import AsyncRawClustersClient, RawClustersClient
from .types.clusters_delete_response import ClustersDeleteResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ClustersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawClustersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawClustersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawClustersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Cluster, ListClustersResponse]:
        """
        Retrieves a list of all latest Clusters. Pagination is available based on query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Cluster, ListClustersResponse]
            Retrieve latest Clusters. If pagination parameters are provided, the response includes paginated data.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        response = client.clusters.list(
            limit=10,
            offset=0,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(limit=limit, offset=offset, request_options=request_options)

    def create_or_update(
        self,
        *,
        manifest: ClusterManifest,
        dry_run: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetClusterResponse:
        """
        Create or Update cluster with provided manifest

        Parameters
        ----------
        manifest : ClusterManifest
            Cluster manifest

        dry_run : typing.Optional[bool]
            Dry run the cluster creation/update

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetClusterResponse
            Returns newly created/updated cluster on success

        Examples
        --------
        from truefoundry_sdk import (
            ClusterManifest,
            ClusterManifestClusterType,
            Collaborator,
            TrueFoundry,
        )

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.clusters.create_or_update(
            manifest=ClusterManifest(
                name="name",
                cluster_type=ClusterManifestClusterType.AWS_EKS,
                environment_names=["environment_names"],
                collaborators=[
                    Collaborator(
                        subject="subject",
                        role_id="role_id",
                    )
                ],
            ),
        )
        """
        _response = self._raw_client.create_or_update(
            manifest=manifest, dry_run=dry_run, request_options=request_options
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetClusterResponse:
        """
        Get cluster associated with provided id

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetClusterResponse
            Return the cluster associated with provided id

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.clusters.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ClustersDeleteResponse:
        """
        Delete cluster associated with provided cluster id

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClustersDeleteResponse
            Returns success message on successful deletion

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.clusters.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def get_addons(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListClusterAddonsResponse:
        """
        List addons for the provided cluster. Pagination is available based on query parameters.

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListClusterAddonsResponse
            Returns a paginated list of addons for the cluster

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.clusters.get_addons(
            id="id",
            limit=10,
            offset=0,
        )
        """
        _response = self._raw_client.get_addons(id, limit=limit, offset=offset, request_options=request_options)
        return _response.data

    def is_connected(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> IsClusterConnectedResponse:
        """
        Get the status of provided cluster

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IsClusterConnectedResponse
            Returns the status of provided cluster

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.clusters.is_connected(
            id="id",
        )
        """
        _response = self._raw_client.is_connected(id, request_options=request_options)
        return _response.data


class AsyncClustersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawClustersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawClustersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawClustersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Cluster, ListClustersResponse]:
        """
        Retrieves a list of all latest Clusters. Pagination is available based on query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Cluster, ListClustersResponse]
            Retrieve latest Clusters. If pagination parameters are provided, the response includes paginated data.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            response = await client.clusters.list(
                limit=10,
                offset=0,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(limit=limit, offset=offset, request_options=request_options)

    async def create_or_update(
        self,
        *,
        manifest: ClusterManifest,
        dry_run: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetClusterResponse:
        """
        Create or Update cluster with provided manifest

        Parameters
        ----------
        manifest : ClusterManifest
            Cluster manifest

        dry_run : typing.Optional[bool]
            Dry run the cluster creation/update

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetClusterResponse
            Returns newly created/updated cluster on success

        Examples
        --------
        import asyncio

        from truefoundry_sdk import (
            AsyncTrueFoundry,
            ClusterManifest,
            ClusterManifestClusterType,
            Collaborator,
        )

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.clusters.create_or_update(
                manifest=ClusterManifest(
                    name="name",
                    cluster_type=ClusterManifestClusterType.AWS_EKS,
                    environment_names=["environment_names"],
                    collaborators=[
                        Collaborator(
                            subject="subject",
                            role_id="role_id",
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update(
            manifest=manifest, dry_run=dry_run, request_options=request_options
        )
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetClusterResponse:
        """
        Get cluster associated with provided id

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetClusterResponse
            Return the cluster associated with provided id

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.clusters.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClustersDeleteResponse:
        """
        Delete cluster associated with provided cluster id

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClustersDeleteResponse
            Returns success message on successful deletion

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.clusters.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def get_addons(
        self,
        id: str,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListClusterAddonsResponse:
        """
        List addons for the provided cluster. Pagination is available based on query parameters.

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListClusterAddonsResponse
            Returns a paginated list of addons for the cluster

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.clusters.get_addons(
                id="id",
                limit=10,
                offset=0,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_addons(id, limit=limit, offset=offset, request_options=request_options)
        return _response.data

    async def is_connected(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> IsClusterConnectedResponse:
        """
        Get the status of provided cluster

        Parameters
        ----------
        id : str
            Cluster id of the cluster

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IsClusterConnectedResponse
            Returns the status of provided cluster

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.clusters.is_connected(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.is_connected(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class SpanFieldFilterSpanFieldName(enum.StrEnum):
    SPAN_NAME = "spanName"
    SPAN_ID = "spanId"
    TRACE_ID = "traceId"
    PARENT_SPAN_ID = "parentSpanId"
    APPLICATION_NAME = "applicationName"
    CREATED_BY_SUBJECT_SLUG = "createdBySubjectSlug"
    CREATED_BY_SUBJECT_TYPE = "createdBySubjectType"
    DURATION_IN_NS = "durationInNs"
    DURATION_IN_MS = "durationInMs"
    SPAN_STATUS_CODE = "spanStatusCode"
    TAG = "tag"
    SCOPE_NAME = "scopeName"
    ENVIRONMENT = "environment"

    def visit(
        self,
        span_name: typing.Callable[[], T_Result],
        span_id: typing.Callable[[], T_Result],
        trace_id: typing.Callable[[], T_Result],
        parent_span_id: typing.Callable[[], T_Result],
        application_name: typing.Callable[[], T_Result],
        created_by_subject_slug: typing.Callable[[], T_Result],
        created_by_subject_type: typing.Callable[[], T_Result],
        duration_in_ns: typing.Callable[[], T_Result],
        duration_in_ms: typing.Callable[[], T_Result],
        span_status_code: typing.Callable[[], T_Result],
        tag: typing.Callable[[], T_Result],
        scope_name: typing.Callable[[], T_Result],
        environment: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is SpanFieldFilterSpanFieldName.SPAN_NAME:
            return span_name()
        if self is SpanFieldFilterSpanFieldName.SPAN_ID:
            return span_id()
        if self is SpanFieldFilterSpanFieldName.TRACE_ID:
            return trace_id()
        if self is SpanFieldFilterSpanFieldName.PARENT_SPAN_ID:
            return parent_span_id()
        if self is SpanFieldFilterSpanFieldName.APPLICATION_NAME:
            return application_name()
        if self is SpanFieldFilterSpanFieldName.CREATED_BY_SUBJECT_SLUG:
            return created_by_subject_slug()
        if self is SpanFieldFilterSpanFieldName.CREATED_BY_SUBJECT_TYPE:
            return created_by_subject_type()
        if self is SpanFieldFilterSpanFieldName.DURATION_IN_NS:
            return duration_in_ns()
        if self is SpanFieldFilterSpanFieldName.DURATION_IN_MS:
            return duration_in_ms()
        if self is SpanFieldFilterSpanFieldName.SPAN_STATUS_CODE:
            return span_status_code()
        if self is SpanFieldFilterSpanFieldName.TAG:
            return tag()
        if self is SpanFieldFilterSpanFieldName.SCOPE_NAME:
            return scope_name()
        if self is SpanFieldFilterSpanFieldName.ENVIRONMENT:
            return environment()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .data_directory_manifest_source import DataDirectoryManifestSource


class DataDirectoryManifest(UniversalBaseModel):
    """
    Data Directory manifest.
    """

    type: typing.Literal["data-dir"] = "data-dir"
    name: str = pydantic.Field()
    """
    Name of the data directory
    """

    ml_repo: str = pydantic.Field()
    """
    Name of the ML Repo to create the data directory under
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description for the data directory
    """

    metadata: typing.Dict[str, typing.Any] = pydantic.Field()
    """
    Key Value metadata. Should be valid JSON. For e.g. `{"business-unit": "sales", "quality": "good", "rating": 4.5}`
    """

    source: DataDirectoryManifestSource

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .parameters_stop import ParametersStop


class Parameters(UniversalBaseModel):
    """
    Parameters to pass to the model when generating
    """

    max_tokens: typing.Optional[int] = None
    temperature: typing.Optional[float] = None
    top_k: typing.Optional[float] = None
    top_p: typing.Optional[float] = None
    stop: typing.Optional[ParametersStop] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class OtelExporterHttpConfigEncoding(enum.StrEnum):
    """
    Encoding for OpenTelemetry data
    """

    PROTO = "proto"
    JSON = "json"

    def visit(self, proto: typing.Callable[[], T_Result], json: typing.Callable[[], T_Result]) -> T_Result:
        if self is OtelExporterHttpConfigEncoding.PROTO:
            return proto()
        if self is OtelExporterHttpConfigEncoding.JSON:
            return json()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class JsonSchema(UniversalBaseModel):
    """
    JSON schema for structured output
    """

    name: str = pydantic.Field()
    """
    Name of the schema
    """

    schema_: typing_extensions.Annotated[typing.Dict[str, typing.Any], FieldMetadata(alias="schema")] = pydantic.Field()
    """
    JSON schema object defining the expected structure
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class MimeType(enum.StrEnum):
    """
    MIME type of the content
    """

    TEXT_PLAIN = "text/plain"
    APPLICATION_JSON = "application/json"
    IMAGE_PNG = "image/png"
    IMAGE_JPEG = "image/jpeg"
    APPLICATION_X_DIRECTORY = "application/x-directory"

    def visit(
        self,
        text_plain: typing.Callable[[], T_Result],
        application_json: typing.Callable[[], T_Result],
        image_png: typing.Callable[[], T_Result],
        image_jpeg: typing.Callable[[], T_Result],
        application_x_directory: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is MimeType.TEXT_PLAIN:
            return text_plain()
        if self is MimeType.APPLICATION_JSON:
            return application_json()
        if self is MimeType.IMAGE_PNG:
            return image_png()
        if self is MimeType.IMAGE_JPEG:
            return image_jpeg()
        if self is MimeType.APPLICATION_X_DIRECTORY:
            return application_x_directory()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class LocalSource(UniversalBaseModel):
    """
    Describes that we are using code stored in a local developement environment to build our image
    """

    type: typing.Literal["local"] = pydantic.Field(default="local")
    """
    +value=local
    """

    project_root_path: str = pydantic.Field(default="./")
    """
    Local project root path.
    """

    local_build: bool = pydantic.Field(default=True)
    """
    run docker build locally
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .custom_basic_auth import CustomBasicAuth
from .custom_bearer_auth import CustomBearerAuth

SelfHostedModelAuthData = typing.Union[CustomBasicAuth, CustomBearerAuth]



# This file was auto-generated by Fern from our API Definition.

import typing

from .fast_ai_framework import FastAiFramework
from .gluon_framework import GluonFramework
from .h2o_framework import H2OFramework
from .keras_framework import KerasFramework
from .light_gbm_framework import LightGbmFramework
from .onnx_framework import OnnxFramework
from .paddle_framework import PaddleFramework
from .py_torch_framework import PyTorchFramework
from .sklearn_framework import SklearnFramework
from .spa_cy_framework import SpaCyFramework
from .stats_models_framework import StatsModelsFramework
from .tensor_flow_framework import TensorFlowFramework
from .transformers_framework import TransformersFramework
from .xg_boost_framework import XgBoostFramework

ModelManifestFramework = typing.Union[
    TransformersFramework,
    TensorFlowFramework,
    SklearnFramework,
    PyTorchFramework,
    KerasFramework,
    XgBoostFramework,
    LightGbmFramework,
    FastAiFramework,
    H2OFramework,
    OnnxFramework,
    SpaCyFramework,
    StatsModelsFramework,
    GluonFramework,
    PaddleFramework,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_charts_response import GetChartsResponse
from .raw_client import AsyncRawMetricsClient, RawMetricsClient
from .types.metrics_get_charts_request_filter_entity import MetricsGetChartsRequestFilterEntity


class MetricsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMetricsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMetricsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMetricsClient
        """
        return self._raw_client

    def get_charts(
        self,
        workspace_id: str,
        *,
        application_id: str,
        filter_entity: MetricsGetChartsRequestFilterEntity,
        start_ts: typing.Optional[str] = None,
        end_ts: typing.Optional[str] = None,
        filter_query: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetChartsResponse:
        """
        List charts for a given Application based on parameters passed in the query.

        Parameters
        ----------
        workspace_id : str

        application_id : str

        filter_entity : MetricsGetChartsRequestFilterEntity

        start_ts : typing.Optional[str]
            Start Timestamp

        end_ts : typing.Optional[str]
            End Timestamp

        filter_query : typing.Optional[str]
            Query params to filter metrics

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetChartsResponse
            Charts have been successfully retrieved.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry
        from truefoundry_sdk.internal.metrics import MetricsGetChartsRequestFilterEntity

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.metrics.get_charts(
            workspace_id="workspaceId",
            application_id="applicationId",
            start_ts="startTs",
            end_ts="endTs",
            filter_entity=MetricsGetChartsRequestFilterEntity.APPLICATION,
            filter_query="filterQuery",
        )
        """
        _response = self._raw_client.get_charts(
            workspace_id,
            application_id=application_id,
            filter_entity=filter_entity,
            start_ts=start_ts,
            end_ts=end_ts,
            filter_query=filter_query,
            request_options=request_options,
        )
        return _response.data


class AsyncMetricsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMetricsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMetricsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMetricsClient
        """
        return self._raw_client

    async def get_charts(
        self,
        workspace_id: str,
        *,
        application_id: str,
        filter_entity: MetricsGetChartsRequestFilterEntity,
        start_ts: typing.Optional[str] = None,
        end_ts: typing.Optional[str] = None,
        filter_query: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetChartsResponse:
        """
        List charts for a given Application based on parameters passed in the query.

        Parameters
        ----------
        workspace_id : str

        application_id : str

        filter_entity : MetricsGetChartsRequestFilterEntity

        start_ts : typing.Optional[str]
            Start Timestamp

        end_ts : typing.Optional[str]
            End Timestamp

        filter_query : typing.Optional[str]
            Query params to filter metrics

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetChartsResponse
            Charts have been successfully retrieved.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry
        from truefoundry_sdk.internal.metrics import MetricsGetChartsRequestFilterEntity

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.metrics.get_charts(
                workspace_id="workspaceId",
                application_id="applicationId",
                start_ts="startTs",
                end_ts="endTs",
                filter_entity=MetricsGetChartsRequestFilterEntity.APPLICATION,
                filter_query="filterQuery",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_charts(
            workspace_id,
            application_id=application_id,
            filter_entity=filter_entity,
            start_ts=start_ts,
            end_ts=end_ts,
            filter_query=filter_query,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .docker_file_build_command import DockerFileBuildCommand


class DockerFileBuild(UniversalBaseModel):
    """
    Describes that we are using a dockerfile to build our image
    """

    type: typing.Literal["dockerfile"] = pydantic.Field(default="dockerfile")
    """
    +value=dockerfile
    """

    dockerfile_path: str = pydantic.Field(default="./Dockerfile")
    """
    The file path of the Dockerfile relative to project root path.
    """

    build_context_path: str = pydantic.Field(default="./")
    """
    Build context path for the Dockerfile relative to project root path.
    """

    command: typing.Optional[DockerFileBuildCommand] = pydantic.Field(default=None)
    """
    Override the command to run when the container starts
    When deploying a Job, the command can be templatized by defining `params` and referencing them in command
    E.g. `python main.py --learning_rate {{learning_rate}}`
    """

    build_args: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Build arguments to pass to docker build
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .notebook_config import NotebookConfig
from .ssh_server_config import SshServerConfig


class ClusterManifestWorkbenchConfig(UniversalBaseModel):
    """
    Workbench Config
    """

    notebook_config: typing.Optional[NotebookConfig] = None
    ssh_server_config: typing.Optional[SshServerConfig] = None
    default_storage_class: typing.Optional[str] = pydantic.Field(default=None)
    """
    The default storage class for the home directory of workbench
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .node_selector import NodeSelector
from .nodepool_selector import NodepoolSelector

ResourcesNode = typing.Union[NodeSelector, NodepoolSelector]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ArtifactType(enum.StrEnum):
    """
    An enumeration.
    """

    ARTIFACT = "artifact"
    MODEL = "model"
    PLOT = "plot"
    IMAGE = "image"
    CHAT_PROMPT = "chat_prompt"

    def visit(
        self,
        artifact: typing.Callable[[], T_Result],
        model: typing.Callable[[], T_Result],
        plot: typing.Callable[[], T_Result],
        image: typing.Callable[[], T_Result],
        chat_prompt: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ArtifactType.ARTIFACT:
            return artifact()
        if self is ArtifactType.MODEL:
            return model()
        if self is ArtifactType.PLOT:
            return plot()
        if self is ArtifactType.IMAGE:
            return image()
        if self is ArtifactType.CHAT_PROMPT:
            return chat_prompt()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .guardrails_when import GuardrailsWhen


class GuardrailsRule(UniversalBaseModel):
    """
    Guardrails Rule
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the rule
    """

    when: GuardrailsWhen
    input_guardrails: typing.List[str] = pydantic.Field()
    """
    Guardrail selectors of the guardrails to apply to the input (format: groupName/guardrailName)
    """

    output_guardrails: typing.List[str] = pydantic.Field()
    """
    Guardrail selectors of the guardrails to apply to the output (format: groupName/guardrailName)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .node_selector_capacity_type import NodeSelectorCapacityType


class NodeSelector(UniversalBaseModel):
    """
    Constraints to select a Node - Specific GPU / Instance Families, On-Demand/Spot.
    """

    type: typing.Literal["node_selector"] = pydantic.Field(default="node_selector")
    """
    +value=node_selector
    """

    instance_families: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Instance family of the underlying machine to use. Multiple instance families can be supplied.
    The workload is guaranteed to be scheduled on one of them.
    """

    capacity_type: typing.Optional[NodeSelectorCapacityType] = pydantic.Field(default=None)
    """
    Configure what type of nodes to run the app. By default no placement logic is applied.
    "spot_fallback_on_demand" will try to place the application on spot nodes but will fallback to on-demand when spot nodes are not available.
    "spot" will strictly place the application on spot nodes.
    "on_demand" will strictly place the application on on-demand nodes.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class MetricsGetChartsRequestFilterEntity(enum.StrEnum):
    APPLICATION = "application"
    POD = "pod"
    JOB_RUN = "job-run"
    EVENT = "event"
    CLUSTER = "cluster"

    def visit(
        self,
        application: typing.Callable[[], T_Result],
        pod: typing.Callable[[], T_Result],
        job_run: typing.Callable[[], T_Result],
        event: typing.Callable[[], T_Result],
        cluster: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is MetricsGetChartsRequestFilterEntity.APPLICATION:
            return application()
        if self is MetricsGetChartsRequestFilterEntity.POD:
            return pod()
        if self is MetricsGetChartsRequestFilterEntity.JOB_RUN:
            return job_run()
        if self is MetricsGetChartsRequestFilterEntity.EVENT:
            return event()
        if self is MetricsGetChartsRequestFilterEntity.CLUSTER:
            return cluster()



# This file was auto-generated by Fern from our API Definition.

import typing

from .local_source import LocalSource
from .remote_source import RemoteSource

WorkflowSource = typing.Union[LocalSource, RemoteSource]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from ..core.serialization import FieldMetadata


class SecretVersion(UniversalBaseModel):
    id: str
    fqn: str
    value: typing.Optional[str] = None
    version: typing.Optional[float] = None
    secret: typing.Optional["Secret"] = None
    secret_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="secretId")] = None
    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = None
    updated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="updatedAt")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


from .secret import Secret  # noqa: E402, I001

update_forward_refs(SecretVersion, Secret=Secret)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ClouderaTokenAuth(UniversalBaseModel):
    """
    Authenticate using a pre-generated API token. Alternative to workload authentication, this method uses a directly provided API token for accessing Cloudera CDP services.
    """

    type: typing.Literal["token"] = pydantic.Field(default="token")
    """
    +value=token
    """

    api_token: str = pydantic.Field()
    """
    Pre-generated API token for Cloudera CDP authentication. You can generate this token using the CDP CLI or from your user profile settings. This token provides direct access to CDP services without requiring workload-specific authentication.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow
